<?php

namespace Drupal\foundation_anchor_menu_block\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;

/**
 * Provides a dynamic 'Foundation Anchor Menu' Block.
 *
 * @Block(
 *   id = "foundation_anchor_menu_block_block",
 *   admin_label = @Translation("Foundation Anchor Menu Block (Dynamic by JavaScript)"),
 *   admin_description = @Translation("Only visible if at least one anchor menu item exists on the current page. Content created at runtime by JavaScript."),
 *   category = @Translation("Navigation"),
 * )
 */
class FoundationAnchorMenuBlock extends BlockBase implements BlockPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function build() {
    return [
      '#theme' => 'famb_anchor_menu',
      '#menu_orientation' => $this->configuration['menu_orientation'],
      '#menu_horizontal_breakpoint' => $this->configuration['menu_horizontal_breakpoint'],
      '#list_classes_additional' => array_unique(explode(' ', $this->configuration['list_classes_additional'])),
      '#attached' => [
        'library' => [
          'foundation_anchor_menu_block/famb-anchor-menu',
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'menu_orientation' => 'horizontal',
      'menu_horizontal_breakpoint' => 'medium',
      'list_classes_additional' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    // Retrieve existing configuration for this block.
    $config = $this->getConfiguration();
    $form['menu_orientation'] = [
      '#type' => 'select',
      '#title' => t('Menu orientation'),
      '#options' => [
        'vertical' => $this->t('Vertical'),
        'horizontal' => $this->t('Horizontal'),
      ],
      '#required' => TRUE,
      '#default_value' => isset($config['menu_orientation']) ? $config['menu_orientation'] : 'horizontal',
      '#description' => $this->t('See <a href="@foundation_docs_vertical_menu_href" target="_blank">Foundation Docs: Vertical Menu</a>', ['@foundation_docs_vertical_menu_href' => 'https://get.foundation/sites/docs/menu.html#vertical-menu']),
    ];
    $form['menu_horizontal_breakpoint'] = [
      '#type' => 'select',
      '#title' => t('Menu horizontal breakpoint'),
      '#options' => [
        'small' => $this->t('Small'),
        'medium' => $this->t('Medium') . ' (' . $this->t('Default') . ')',
        'large' => $this->t('Large'),
      ],
      '#required' => TRUE,
      '#default_value' => isset($config['menu_horizontal_breakpoint']) ? $config['menu_horizontal_breakpoint'] : 'medium',
      '#description' => $this->t('If the screen is smaller than this breakpoint, switches a horizontal menu to vertical.'),
    ];
    $form['list_classes_additional'] = [
      '#type' => 'textfield',
      '#title' => t('Additional list classes'),
      '#required' => FALSE,
      '#default_value' => isset($config['list_classes_additional']) ? $config['list_classes_additional'] : '',
      '#description' => $this->t('Adds further custom classes to the list (&lt;ul&gt;) element. Separated by space.'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    // Save our custom settings when the form is submitted.
    $this->setConfigurationValue('menu_orientation', $form_state->getValue('menu_orientation'));
    $this->setConfigurationValue('menu_horizontal_breakpoint', $form_state->getValue('menu_horizontal_breakpoint'));
    $this->setConfigurationValue('list_classes_additional', Html::escape(trim($form_state->getValue('list_classes_additional'))));
  }

  /**
   * {@inheritdoc}
   */
  public function blockValidate($form, FormStateInterface $form_state) {
    $menu_orientation = $form_state->getValue('menu_orientation');
    if (!in_array($menu_orientation, ['vertical', 'horizontal'])) {
      $form_state->setErrorByName('menu_orientation', $this->t('Must be @vertical or @horizontal', ['@vertical' => $this->t('Vertical'), '@horizontal' => $this->t('Horizontal')]));
    }

    $menu_horizontal_breakpoint = $form_state->getValue('menu_horizontal_breakpoint');
    if (!in_array($menu_horizontal_breakpoint, ['small', 'medium', 'large'])) {
      $form_state->setErrorByName('menu_horizontal_breakpoint', $this->t('Must be small, medium or large.'));
    }

    $menu_horizontal_breakpoint = $form_state->getValue('menu_horizontal_breakpoint');
    if (!in_array($menu_horizontal_breakpoint, ['small', 'medium', 'large'])) {
      $form_state->setErrorByName('menu_horizontal_breakpoint', $this->t('Must be small, medium or large.'));
    }
  }

}
